# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-06-03 08:12
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='book',
            name='genre',
            field=models.PositiveIntegerField(choices=[(1, 'Drama'), (2, 'Comics'), (3, 'Other')], null=True, blank=True),
        ),
    ]
