# Generated by Django 2.1.2 on 2018-11-20 20:58

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0003_book_similar_books'),
    ]

    operations = [
        migrations.AlterField(
            model_name='book',
            name='content_type',
            field=models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType'),
        ),
        migrations.AlterField(
            model_name='book',
            name='object_id',
            field=models.PositiveIntegerField(editable=False, null=True),
        ),
        migrations.AlterField(
            model_name='book',
            name='similar_books',
            field=models.ManyToManyField(blank=True, related_name='_book_similar_books_+', to='core.Book'),
        ),
    ]
